@@ -95,24 +95,26 @@ test_that("Wheat_asreml4", {
   testthat::expect_equal(ncol(Var.diffs$differences), 25)
   testthat::expect_equal(nrow(Var.diffs$p.differences), 25)
   testthat::expect_equal(ncol(Var.diffs$p.differences), 25)
-  testthat::expect_equal(length(Var.diffs$LSD), 3)
+  testthat::expect_equal(length(Var.diffs$LSD), 8)
   testthat::expect_true("lower.halfLeastSignificant.limit" %in% names(Var.diffs$predictions))
   testthat::expect_equal(Var.diffs$backtransforms, NULL)
   testthat::expect_equal(as.character(Var.diffs$predictions$Variety[[1]]),"10")
   testthat::expect_silent(plotPvalues(Var.diffs))
+  testthat::expect_silent(plotPvalues(Var.diffs, show.sig = TRUE, alpha = 0.05))
   
   #Test for single-value LSDs
   diffs <- predictPlus(classify = "Variety", 
                            asreml.obj=current.asr, 
                            error.intervals="halfLeast",
-                           meanLSD.type = "fact", LSDby = "Variety",
+                           LSDtype = "fact", LSDby = "Variety",
                            wald.tab=current.asrt$wald.tab,
                            tables = "predictions", 
                            sortFactor = "Variety")
   testthat::expect_equal(nrow(diffs$LSD), 25)
   testthat::expect_true("lower.halfLeastSignificant.limit" %in% names(diffs$predictions))
+  #Are the LSDs sorted?
   testthat::expect_true(all((diffs$predictions$upper.halfLeastSignificant.limit - 
                                diffs$predictions$lower.halfLeastSignificant.limit - 
-                               diffs$LSD$meanLSD[as.numfac(diffs$predictions$Variety)]) < 1e-05))
+                               diffs$LSD$meanLSD) < 1e-05))
   diffs$predictions$upper.halfLeastSignificant.limit - diffs$predictions$lower.halfLeastSignificant.limit
 })
